@@ -6,6 +6,8 @@ from collections import OrderedDict
 from enum import Enum
 from typing import Any, Dict, List, Optional, Union, cast
 
+from pydantic import BaseModel, ConfigDict
+
 from langroid.agent.tool_message import ToolMessage
 from langroid.agent.xml_tool_message import XMLToolMessage
 from langroid.language_models.base import (
@@ -21,7 +23,6 @@ from langroid.mytypes import DocMetaData, Document, Entity
 from langroid.parsing.agent_chats import parse_message
 from langroid.parsing.file_attachment import FileAttachment
 from langroid.parsing.parse_json import extract_top_level_json, top_level_json_field
-from langroid.pydantic_v1 import BaseModel, Extra
 from langroid.utils.object_registry import ObjectRegistry
 from langroid.utils.output.printing import shorten_text
 from langroid.utils.types import to_string
@@ -29,8 +30,7 @@ from langroid.utils.types import to_string
 
 class ChatDocAttachment(BaseModel):
     # any additional data that should be attached to the document
-    class Config:
-        extra = Extra.allow
+    model_config = ConfigDict(extra="allow")
 
 
 class StatusCode(str, Enum):
@@ -89,7 +89,7 @@ class ChatDocLoggerFields(BaseModel):
 
     @classmethod
     def tsv_header(cls) -> str:
-        field_names = cls().dict().keys()
+        field_names = cls().model_dump().keys()
         return "\t".join(field_names)
 
 
@@ -217,25 +217,29 @@ class ChatDocument(Document):
         """
         tool_type = ""  # FUNC or TOOL
         tool = ""  # tool name or function name
-        oai_tools = (
-            []
-            if self.oai_tool_calls is None
-            else [t for t in self.oai_tool_calls if t.function is not None]
-        )
-        if self.function_call is not None:
-            tool_type = "FUNC"
-            tool = self.function_call.name
-        elif len(oai_tools) > 0:
-            tool_type = "OAI_TOOL"
-            tool = ",".join(t.function.name for t in oai_tools)  # type: ignore
-        else:
-            try:
-                json_tools = self.get_tool_names()
-            except Exception:
-                json_tools = []
-            if json_tools != []:
-                tool_type = "TOOL"
-                tool = json_tools[0]
+
+        # Skip tool detection for system messages - they contain tool instructions,
+        # not actual tool calls
+        if self.metadata.sender != Entity.SYSTEM:
+            oai_tools = (
+                []
+                if self.oai_tool_calls is None
+                else [t for t in self.oai_tool_calls if t.function is not None]
+            )
+            if self.function_call is not None:
+                tool_type = "FUNC"
+                tool = self.function_call.name
+            elif len(oai_tools) > 0:
+                tool_type = "OAI_TOOL"
+                tool = ",".join(t.function.name for t in oai_tools)  # type: ignore
+            else:
+                try:
+                    json_tools = self.get_tool_names()
+                except Exception:
+                    json_tools = []
+                if json_tools != []:
+                    tool_type = "TOOL"
+                    tool = json_tools[0]
         recipient = self.metadata.recipient
         content = self.content
         sender_entity = self.metadata.sender
@@ -255,7 +259,7 @@ class ChatDocument(Document):
     def tsv_str(self) -> str:
         fields = self.log_fields()
         fields.content = shorten_text(fields.content, 80)
-        field_values = fields.dict().values()
+        field_values = fields.model_dump().values()
         return "\t".join(str(v) for v in field_values)
 
     def pop_tool_ids(self) -> None:
@@ -340,6 +344,50 @@ class ChatDocument(Document):
             ),
         )
 
+    @staticmethod
+    def from_LLMMessage(
+        message: LLMMessage,
+        sender_name: str = "",
+        recipient: str = "",
+    ) -> "ChatDocument":
+        """
+        Convert LLMMessage to ChatDocument.
+
+        Args:
+            message (LLMMessage): LLMMessage to convert.
+            sender_name (str): Name of the sender. Defaults to "".
+            recipient (str): Name of the recipient. Defaults to "".
+
+        Returns:
+            ChatDocument: ChatDocument representation of this LLMMessage.
+        """
+        # Map LLMMessage Role to ChatDocument Entity
+        role_to_entity = {
+            Role.USER: Entity.USER,
+            Role.SYSTEM: Entity.SYSTEM,
+            Role.ASSISTANT: Entity.LLM,
+            Role.FUNCTION: Entity.LLM,
+            Role.TOOL: Entity.LLM,
+        }
+
+        sender_entity = role_to_entity.get(message.role, Entity.USER)
+
+        return ChatDocument(
+            content=message.content or "",
+            content_any=message.content,
+            files=message.files,
+            function_call=message.function_call,
+            oai_tool_calls=message.tool_calls,
+            metadata=ChatDocMetaData(
+                source=sender_entity,
+                sender=sender_entity,
+                sender_name=sender_name,
+                recipient=recipient,
+                oai_tool_id=message.tool_call_id,
+                tool_ids=[message.tool_id] if message.tool_id else [],
+            ),
+        )
+
     @staticmethod
     def to_LLMMessage(
         message: Union[str, "ChatDocument"],
@@ -462,5 +510,5 @@ class ChatDocument(Document):
         ]
 
 
-LLMMessage.update_forward_refs()
-ChatDocMetaData.update_forward_refs()
+LLMMessage.model_rebuild()
+ChatDocMetaData.model_rebuild()
