@@ -8,7 +8,8 @@ NOTE: Using this tool requires setting the GOOGLE_API_KEY and GOOGLE_CSE_ID
 environment variables in your `.env` file, as explained in the
 [README](https://github.com/langroid/langroid#gear-installation-and-setup).
 """
-from typing import List
+
+from typing import List, Tuple
 
 from langroid.agent.tool_message import ToolMessage
 from langroid.parsing.web_search import google_search
@@ -29,7 +30,7 @@ class GoogleSearchTool(ToolMessage):
         return "\n\n".join(str(result) for result in search_results)
 
     @classmethod
-    def examples(cls) -> List["ToolMessage"]:
+    def examples(cls) -> List["ToolMessage" | Tuple[str, "ToolMessage"]]:
         return [
             cls(
                 query="When was the Llama2 Large Language Model (LLM) released?",
