@@ -7,15 +7,14 @@ from typing import Any, Dict, List, TypeVar
 import fakeredis
 import redis
 from dotenv import load_dotenv
-from pydantic import BaseModel
 
-from langroid.cachedb.base import CacheDB
+from langroid.cachedb.base import CacheDB, CacheDBConfig
 
 T = TypeVar("T", bound="RedisCache")
 logger = logging.getLogger(__name__)
 
 
-class RedisCacheConfig(BaseModel):
+class RedisCacheConfig(CacheDBConfig):
     """Configuration model for RedisCache."""
 
     fake: bool = False
@@ -24,6 +23,8 @@ class RedisCacheConfig(BaseModel):
 class RedisCache(CacheDB):
     """Redis implementation of the CacheDB."""
 
+    _warned_password: bool = False
+
     def __init__(self, config: RedisCacheConfig):
         """
         Initialize a RedisCache with the given config.
@@ -41,17 +42,19 @@ class RedisCache(CacheDB):
             redis_host = os.getenv("REDIS_HOST")
             redis_port = os.getenv("REDIS_PORT")
             if None in [redis_password, redis_host, redis_port]:
-                logger.warning(
-                    """REDIS_PASSWORD, REDIS_HOST, REDIS_PORT not set in .env file,
-                    using fake redis client"""
-                )
+                if not RedisCache._warned_password:
+                    logger.warning(
+                        """REDIS_PASSWORD, REDIS_HOST, REDIS_PORT not set in .env file,
+                        using fake redis client"""
+                    )
+                    RedisCache._warned_password = True
                 self.pool = fakeredis.FakeStrictRedis()  # type: ignore
             else:
                 self.pool = redis.ConnectionPool(  # type: ignore
                     host=redis_host,
                     port=redis_port,
                     password=redis_password,
-                    max_connections=50,
+                    max_connections=500,
                     socket_timeout=5,
                     socket_keepalive=True,
                     retry_on_timeout=True,
@@ -109,7 +112,7 @@ class RedisCache(CacheDB):
             key (str): The key to retrieve the value for.
 
         Returns:
-            dict: The value associated with the key.
+            dict|str|None: The value associated with the key.
         """
         with self.redis_client() as client:  # type: ignore
             try:
