@@ -4,6 +4,7 @@ from typing import Callable
 from dotenv import load_dotenv
 from httpx import Timeout
 from openai import AsyncAzureOpenAI, AzureOpenAI
+from pydantic_settings import SettingsConfigDict
 
 from langroid.language_models.openai_gpt import (
     OpenAIGPT,
@@ -56,8 +57,7 @@ class AzureConfig(OpenAIGPTConfig):
     # AZURE_OPENAI_API_VERSION=2023-05-15
     # This is either done in the .env file, or via an explicit
     # `export AZURE_OPENAI_API_VERSION=...`
-    class Config:
-        env_prefix = "AZURE_OPENAI_"
+    model_config = SettingsConfigDict(env_prefix="AZURE_OPENAI_")
 
     def __init__(self, **kwargs) -> None:  # type: ignore
         if "model_name" in kwargs and "chat_model" not in kwargs:
