@@ -1,7 +1,7 @@
 from enum import Enum
 from typing import Dict, List, Optional
 
-from langroid.pydantic_v1 import BaseModel
+from pydantic import BaseModel
 
 
 class ModelProvider(str, Enum):
@@ -36,6 +36,9 @@ class OpenAIChatModel(ModelName):
     GPT4_1 = "gpt-4.1"
     GPT4_1_MINI = "gpt-4.1-mini"
     GPT4_1_NANO = "gpt-4.1-nano"
+    GPT5 = "gpt-5"
+    GPT5_MINI = "gpt-5-mini"
+    GPT5_NANO = "gpt-5-nano"
 
 
 class OpenAICompletionModel(str, Enum):
@@ -71,10 +74,10 @@ class GeminiModel(ModelName):
     GEMINI_1_5_PRO = "gemini-1.5-pro"
     GEMINI_2_5_PRO = "gemini-2.5-pro"
     GEMINI_2_5_FLASH = "gemini-2.5-flash"
-    GEMINI_2_5_FLASH_LITE_PREVIEW = "gemini-2.5-flash-lite-preview-06-17"
+    GEMINI_2_5_FLASH_LITE = "gemini-2.5-flash-lite"
     GEMINI_2_PRO = "gemini-2.0-pro-exp-02-05"
     GEMINI_2_FLASH = "gemini-2.0-flash"
-    GEMINI_2_FLASH_LITE = "gemini-2.0-flash-lite-preview"
+    GEMINI_2_FLASH_LITE = "gemini-2.0-flash-lite"
     GEMINI_2_FLASH_THINKING = "gemini-2.0-flash-thinking-exp"
 
 
@@ -87,6 +90,9 @@ class OpenAI_API_ParamInfo(BaseModel):
     params: Dict[str, List[str]] = dict(
         reasoning_effort=[
             OpenAIChatModel.O3_MINI.value,
+            GeminiModel.GEMINI_2_5_PRO.value,
+            GeminiModel.GEMINI_2_5_FLASH.value,
+            GeminiModel.GEMINI_2_5_FLASH_LITE.value,
         ],
     )
     # model-specific params in extra_body
@@ -173,6 +179,7 @@ MODEL_INFO: Dict[str, ModelInfo] = {
     OpenAIChatModel.GPT4_1_NANO.value: ModelInfo(
         name=OpenAIChatModel.GPT4_1_NANO.value,
         provider=ModelProvider.OPENAI,
+        has_structured_output=True,
         context_length=1_047_576,
         max_output_tokens=32_768,
         input_cost_per_million=0.10,
@@ -183,6 +190,7 @@ MODEL_INFO: Dict[str, ModelInfo] = {
     OpenAIChatModel.GPT4_1_MINI.value: ModelInfo(
         name=OpenAIChatModel.GPT4_1_MINI.value,
         provider=ModelProvider.OPENAI,
+        has_structured_output=True,
         context_length=1_047_576,
         max_output_tokens=32_768,
         input_cost_per_million=0.40,
@@ -193,6 +201,7 @@ MODEL_INFO: Dict[str, ModelInfo] = {
     OpenAIChatModel.GPT4_1.value: ModelInfo(
         name=OpenAIChatModel.GPT4_1.value,
         provider=ModelProvider.OPENAI,
+        has_structured_output=True,
         context_length=1_047_576,
         max_output_tokens=32_768,
         input_cost_per_million=2.00,
@@ -232,6 +241,7 @@ MODEL_INFO: Dict[str, ModelInfo] = {
         output_cost_per_million=60.0,
         allows_streaming=True,
         allows_system_message=False,
+        has_structured_output=True,
         unsupported_params=["temperature"],
         rename_params={"max_tokens": "max_completion_tokens"},
         has_tools=False,
@@ -247,6 +257,7 @@ MODEL_INFO: Dict[str, ModelInfo] = {
         output_cost_per_million=8.0,
         allows_streaming=True,
         allows_system_message=False,
+        has_structured_output=True,
         unsupported_params=["temperature"],
         rename_params={"max_tokens": "max_completion_tokens"},
         has_tools=False,
@@ -262,6 +273,7 @@ MODEL_INFO: Dict[str, ModelInfo] = {
         output_cost_per_million=4.4,
         allows_streaming=False,
         allows_system_message=False,
+        has_structured_output=True,
         unsupported_params=["temperature", "stream"],
         rename_params={"max_tokens": "max_completion_tokens"},
         has_tools=False,
@@ -277,6 +289,7 @@ MODEL_INFO: Dict[str, ModelInfo] = {
         output_cost_per_million=4.4,
         allows_streaming=False,
         allows_system_message=False,
+        has_structured_output=True,
         unsupported_params=["temperature", "stream"],
         rename_params={"max_tokens": "max_completion_tokens"},
         has_tools=False,
@@ -292,11 +305,60 @@ MODEL_INFO: Dict[str, ModelInfo] = {
         output_cost_per_million=4.40,
         allows_streaming=False,
         allows_system_message=False,
+        has_structured_output=True,
         unsupported_params=["temperature", "stream"],
         rename_params={"max_tokens": "max_completion_tokens"},
         has_tools=False,
         description="O3 Mini Reasoning LM",
     ),
+    OpenAIChatModel.GPT5.value: ModelInfo(
+        name=OpenAIChatModel.GPT5.value,
+        provider=ModelProvider.OPENAI,
+        context_length=400_000,
+        max_output_tokens=128_000,
+        input_cost_per_million=1.25,
+        cached_cost_per_million=0.125,
+        output_cost_per_million=10.00,
+        allows_streaming=False,
+        allows_system_message=False,
+        has_structured_output=True,
+        unsupported_params=["temperature"],
+        rename_params={"max_tokens": "max_completion_tokens"},
+        has_tools=False,
+        description="GPT-5",
+    ),
+    OpenAIChatModel.GPT5_MINI.value: ModelInfo(
+        name=OpenAIChatModel.GPT5_MINI.value,
+        provider=ModelProvider.OPENAI,
+        context_length=400_000,
+        max_output_tokens=128_000,
+        input_cost_per_million=0.25,
+        cached_cost_per_million=0.025,
+        output_cost_per_million=2.00,
+        allows_streaming=False,
+        allows_system_message=False,
+        has_structured_output=True,
+        unsupported_params=["temperature"],
+        rename_params={"max_tokens": "max_completion_tokens"},
+        has_tools=False,
+        description="GPT-5 Mini",
+    ),
+    OpenAIChatModel.GPT5_NANO.value: ModelInfo(
+        name=OpenAIChatModel.GPT5_NANO.value,
+        provider=ModelProvider.OPENAI,
+        context_length=400_000,
+        max_output_tokens=128_000,
+        input_cost_per_million=0.05,
+        cached_cost_per_million=0.005,
+        output_cost_per_million=0.40,
+        allows_streaming=False,
+        allows_system_message=False,
+        has_structured_output=True,
+        unsupported_params=["temperature"],
+        rename_params={"max_tokens": "max_completion_tokens"},
+        has_tools=False,
+        description="GPT-5 Nano",
+    ),
     # Anthropic Models
     AnthropicModel.CLAUDE_3_5_SONNET.value: ModelInfo(
         name=AnthropicModel.CLAUDE_3_5_SONNET.value,
@@ -379,7 +441,7 @@ MODEL_INFO: Dict[str, ModelInfo] = {
         input_cost_per_million=0.075,
         output_cost_per_million=0.30,
         rename_params={"max_tokens": "max_completion_tokens"},
-        description="Gemini 2.0 Flash Lite Preview",
+        description="Gemini 2.0 Flash Lite",
     ),
     GeminiModel.GEMINI_1_5_FLASH.value: ModelInfo(
         name=GeminiModel.GEMINI_1_5_FLASH.value,
@@ -444,8 +506,8 @@ MODEL_INFO: Dict[str, ModelInfo] = {
         rename_params={"max_tokens": "max_completion_tokens"},
         description="Gemini 2.5 Flash",
     ),
-    GeminiModel.GEMINI_2_5_FLASH_LITE_PREVIEW.value: ModelInfo(
-        name=GeminiModel.GEMINI_2_5_FLASH_LITE_PREVIEW.value,
+    GeminiModel.GEMINI_2_5_FLASH_LITE.value: ModelInfo(
+        name=GeminiModel.GEMINI_2_5_FLASH_LITE.value,
         provider=ModelProvider.GOOGLE,
         context_length=65_536,
         max_output_tokens=65_536,
@@ -453,7 +515,7 @@ MODEL_INFO: Dict[str, ModelInfo] = {
         cached_cost_per_million=0.025,
         output_cost_per_million=0.40,
         rename_params={"max_tokens": "max_completion_tokens"},
-        description="Gemini 2.5 Flash Lite Preview",
+        description="Gemini 2.5 Flash Lite",
     ),
 }
 
