@@ -1,9 +1,10 @@
-from .base import PromptFormatter
-from .llama2_formatter import Llama2Formatter
-from ..config import PromptFormatterConfig, Llama2FormatterConfig
-
 from . import base
 from . import llama2_formatter
+from .base import PromptFormatter
+from .llama2_formatter import Llama2Formatter
+from ..config import PromptFormatterConfig
+from ..config import Llama2FormatterConfig
+
 
 __all__ = [
     "PromptFormatter",
