@@ -2,12 +2,12 @@ from functools import reduce
 from typing import Callable, List
 
 import tiktoken
+from pydantic_settings import BaseSettings
 from pygments import lex
 from pygments.lexers import get_lexer_by_name
 from pygments.token import Token
 
 from langroid.mytypes import Document
-from langroid.pydantic_v1 import BaseSettings
 
 
 def chunk_code(
