@@ -1,7 +1,7 @@
 import re
-from typing import List
+from typing import Any, List
 
-from langroid.pydantic_v1 import BaseModel, Field
+from pydantic import BaseModel, Field, field_validator
 
 HEADER_CONTEXT_SEP = "\n...\n"
 
@@ -24,8 +24,8 @@ class Node(BaseModel):
     # Forward references will be resolved with the update_forward_refs call below.
 
 
-# Resolve forward references for Node (required for recursive models in Pydantic v1)
-Node.update_forward_refs()
+# Resolve forward references for Node (required for recursive models in Pydantic)
+Node.model_rebuild()
 
 
 def _cleanup_text(text: str) -> str:
@@ -180,6 +180,16 @@ class MarkdownChunkConfig(BaseModel):
     rollup: bool = True  # whether to roll up chunks
     header_context_sep: str = HEADER_CONTEXT_SEP  # separator for header context
 
+    @field_validator("chunk_size", mode="before")
+    @classmethod
+    def convert_chunk_size_to_int(cls, v: Any) -> int:
+        """Convert chunk_size to int, maintaining backward compatibility
+        with Pydantic V1.
+        """
+        if isinstance(v, float):
+            return int(v)
+        return int(v)
+
 
 # A simple tokenizer that counts tokens as whitespace-separated words.
 def count_words(text: str) -> int:
