@@ -18,10 +18,12 @@ if TYPE_CHECKING:
     from github.Label import Label
     from github.Repository import Repository
 
+from pydantic import BaseModel, Field
+from pydantic_settings import BaseSettings
+
 from langroid.mytypes import DocMetaData, Document
 from langroid.parsing.document_parser import DocumentParser, DocumentType
 from langroid.parsing.parser import Parser, ParsingConfig
-from langroid.pydantic_v1 import BaseModel, BaseSettings, Field
 
 logger = logging.getLogger(__name__)
 
